# nxpy.maven package ---------------------------------------------------------

# Copyright Nicola Musatti 2011 - 2012
# Use, modification, and distribution are subject to the Boost Software
# License, Version 1.0. (See accompanying file LICENSE.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# See http://nxpy.sourceforge.net for library home page. ---------------------

r"""
Tools to drive the Maven build tool and to manipulate its configuration files.

"""
