# nxpy package -------------------------------------------------------

# Copyright Nicola Musatti 2011 - 2017
# Use, modification, and distribution are subject to the Boost Software
# License, Version 1.0. (See accompanying file LICENSE.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# See http://nxpy.sourceforge.net for library home page. ---------------------

r"""
Nxpy is an etherogeneous collection of useful modules, dealing with diverse topics such as 
wrapping complex commands with API's or interacting with Java and .NET build systems. 
There are also more basic utilities, such as automation of backup files, support for writing your 
own file-like objects and many other things.

The library is being developed with Python 3.6 so as to be compatible with Python 2.7. Tests are run
and most modules work also with 2.6, 3.3, 3.4 and 3.5. Some should still work with 3.2 and 2.5.

.. IMPORTANT::
   Nxpy is a personal project and, while I use a good part of its contents on an almost daily basis,
   my use cases are too specific to guarantee that everything works as expected.

The Nxpy logo was drawn by Claudia Romano.

"""
